<?php
/*
Plugin Name: Instagram Auto Poster فارسی
Plugin URI: http://sominus.net/instagram-auto-poster/
Description: The plugin allows to publish in your account instagram your posts.
Version: 2.0.6
Author: 20script
Author URI: http://www.20script.ir
*/

require 'instagram-api/Instagram.php';

$instagramAutoPoster = new InstagramAutoPoster();
$instagramAutoPoster->init();

class InstagramAutoPoster {
    protected $pageName = 'instagram-auto-poster.php';
    protected $settingsGroup = 'instagram_auto_poster_settings';
    protected $domain = 'instagram-auto-poster';

    private $postId = 0;

    public function setPostId($id) {
        $this->postId = $id;
    }
    
    public function init() {
        load_plugin_textdomain($this->domain, false, dirname(plugin_basename(__FILE__)) . '/languages');
           
        add_action('admin_head', array($this, 'admincss'));
        add_action('admin_head', array($this, 'adminjs'));
        add_action('admin_menu', array($this, 'plugin_options'));
        add_action('admin_init', array($this, 'register_settings'));
                 
        add_filter('publish_post', array($this, 'onPostSave'));
        
        add_filter("manage_edit-post_columns", array($this, 'add_column'), 4);
        add_filter("manage_post_posts_custom_column", array($this, 'fill_column'), 5, 2);
        
        add_action('admin_init', array($this, 'addEvents'));
                
        add_action('wp_ajax_instagram_send', array($this, 'ajaxSend'));
        
        add_action('add_meta_boxes', array($this, 'addMetaBox'));
        add_action('instagram_scheduled', array($this, 'publishToInstagram'), 10, 3);
        
        add_filter('plugin_action_links_' . plugin_basename(__FILE__), array($this, 'settings_link'));
    }
    
    public function addEvents() {
        foreach ($this->getSupportPostType() as $post_type) {
            add_filter("publish_{$post_type[name]}", array($this, 'onPostSave'));
            
            add_filter("manage_edit-{$post_type[name]}_columns", array($this, 'add_column'), 4);
            add_filter("manage_{$post_type[name]}_posts_custom_column", array($this, 'fill_column'), 5, 2);
        }
    }
    
    public function getSupportPostType() {
        $result = array();
        $args = array(
            'public' => true,
            '_builtin' => false
        );
        
        foreach (get_post_types($args) as $post_type_name) {
            if (is_post_type_hierarchical($post_type_name)) continue;
                
            $post_type_data = get_post_type_object($post_type_name);
            if ($post_type_data->show_ui === FALSE) continue;
            
            $result[] = array(
                'name' => $post_type_name,
                'data' => $post_type_data
            );
        }
        
        return $result;
    }

    public function settings_link($links) {
        $links[] = '<a href="'. esc_url(get_admin_url(null, 'options-general.php?page=instagram-auto-poster.php')) .'">Settings</a>';
        return $links;
    }
    
    protected function getOption($name) {
        $options = get_option($this->settingsGroup);
        if (empty($options) || !is_array($options)) {
            return null;
        }

        return $options[$name];
    }
    
    public function plugin_options() {
        add_options_page(
            __('Settings - Instagram Auto Poster', $this->domain),
            __('Instagram Auto Poster', $this->domain),
            'manage_options',
            $this->pageName,
            array($this, 'plugin_options_page')
        );
    }
    
    public function plugin_options_page() {
        echo '
        <div class="wrap">
            <h2>' . __('Instagram Auto Poster', $this->domain) . '</h2>
            <p>' . __('The plugin allows to publish in your account instagram your posts.', $this->domain) . '</p>
            
            <form method="post" enctype="multipart/form-data" action="options.php">
        ';
                
        echo settings_fields($this->settingsGroup);
        echo do_settings_sections($this->pageName);
        
        echo '  <p class="submit">
                    <input type="submit" class="button-primary" value="' . __('Save Changes', $this->domain) . '" />
                </p>
            </form>
        </div>
        ';
    }
    
    public function register_settings() {
        register_setting($this->settingsGroup, $this->settingsGroup, array($this, 'true_validate_settings'));

        add_settings_section('instagram_section', __('Settings', $this->domain), '', $this->pageName);
        
        //login
        add_settings_field(
            'login',
            __('Instagram login', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type'      => 'text',
                'id'        => 'login',
                'required'  => 'required="required"',
                'desc'      => '',
                'label_for' => 'login'
            )
        );
        
        //passw
        add_settings_field(
            'passw',
            __('Instagram password', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type'      => 'password',
                'id'        => 'passw',
                'required'  => 'required="required"',
                'desc'      => '',
                'label_for' => 'passw'
            )
        );
                
        //make
        add_settings_field(
            'make',
            __('How to make auto-posts?', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type' => 'radio',
                'id'   => 'make',
                'vals' => array(
                    'cron' => __('Use WP Cron to Schedule autoposts', $this->domain),
                    'publish' => __('Send new posts in instagram when publishing', $this->domain),
                    'not' => __('Do not send automatically', $this->domain),
                )
            )
        );
        
        //crop image
        add_settings_field(
            'crop',
            __('Crop the image ratio 1:1 when sending?', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type' => 'radio',
                'id'   => 'crop',
                'vals' => array(
                    'with_bg' => __('Crop an image in a square with a white background', $this->domain),
                    'on' => __('Crop an image into a 1:1 square', $this->domain),
                    'off' => __('Disable', $this->domain),
                )
            )
        );
        
        //separator
        add_settings_field(
            'separator',
            __('Delimiter tags', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type' => 'radio',
                'id'   => 'separator',
                'vals' => array(
                    'remove' => __('Remove the space on the tags', $this->domain),
                    'dash' => __('Replace the space on the dash', $this->domain),
                    'underscore' => __('Replace the space on the underscore', $this->domain),
                    'not' => __('Do not separate tags', $this->domain),
                )
            )
        );
        
        //shortlink
        add_settings_field(
            'shortlink',
            __('Replace {URL} in the short link?', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type' => 'radio',
                'id'   => 'shortlink',
                'vals' => array(
                    'on' => __('Enable', $this->domain),
                    'off' => __('Disable', $this->domain),
                )
            )
        );

        //post type        
        foreach ($this->getSupportPostType() as $post_type) {
            add_settings_field(
                "post_type_{$post_type[name]}",
                __("Enable post type", $this->domain) . " - " . $post_type['data']->labels->singular_name . "?",
                array($this, 'display_input_field'),
                $this->pageName,
                'instagram_section',
                array(
                    'type' => 'checkbox',
                    'id'   => "post_type_{$post_type[name]}",
                )
            );
        }
        
        $args = array(
            'public' => true,
            '_builtin' => false,
        );
        
        $taxonomies = get_taxonomies($args, 'objects');
        
        $format = array(
            "<div></div>",
            "<b>{TITLE}</b> - Insert the Title of the post",
            "<b>{URL}</b> - Insert the URL of the post",
            "<b>{EXCERPT}</b> - Insert the excerpt of the post",
            "<b>{CONTENT}</b> - Insert the full content of the post",
            "<b>{AUTHORNAME}</b> - Insert the author's name",
            "<b>{SITENAME}</b> - Insert the Blog/Site name",
            "<b>{CATS}</b> - Insert post categories",
            "<b>{HCATS}</b> - Insert post categories as hashtags",
            "<b>{TAGS}</b> - Insert post tags",
            "<b>{HTAGS}</b> - Insert post tags as hashtags",
            "<br>Custom taxonomy<br>"
        );
        
        foreach ($taxonomies as $name => $taxonomy) {
            $format[] = "<b>{".mb_strtoupper($name)."}</b> - Insert $name taxonomy of post";
            $format[] = "<b>{HTAGS_".mb_strtoupper($name)."}</b> - Insert $name taxonomy as hashtags of post";
        }
        
        $format[] = "<br>For WooCommerce product<br>";
        $format[] = "<b>{SKU}</b> - Insert sku code of the post";
        $format[] = "<b>{PRICE}</b> - Insert regular or sale price of the post";

        //htags
        add_settings_field(
            'format',
            __('Message text format', $this->domain),
            array($this, 'display_input_field'),
            $this->pageName,
            'instagram_section',
            array(
                'type' => 'textarea',
                'id'   => 'format',
                'desc' => implode("<br>", $format),
                'label_for' => 'format'
            )
        );
    }
        
    public function true_validate_settings($input) {
        foreach ($input as $k => $v) {
            $valid_input[$k] = trim($v);
        }
        return $valid_input;
    }
    
    public function addMetaBox() {
        add_meta_box('instagram_box', __('Instagram Auto Poster', $this->domain), array($this, 'metaBoxGetPrintCheckResults'), 'post', 'side', 'high');
        foreach ($this->getSupportPostType() as $post_type) {
            add_meta_box('instagram_box', __('Instagram Auto Poster', $this->domain), array($this, 'metaBoxGetPrintCheckResults'), $post_type['name'], 'side', 'high');
        }
    }
    
    public function metaBoxGetPrintCheckResults() {
        global $post;

        wp_nonce_field(plugin_basename(__FILE__), 'boom_noncename');
                
        $status = get_post_meta($post->ID, "instagram-send", true);

        if (is_array($status) && isset($status['status']) && $status['status'] == 'fail') {
            echo '<div class="instagram_date"></div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result">'.$status['message'].'</div>';
        } elseif ((is_array($status) && isset($status['status']) && $status['status'] == 'ok') || $status == "1") {
            echo '<div class="instagram_date">' . $this->getPostedDate($post->ID) . '</div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result"></div>';
        } elseif ($status == "sending") {
            echo '<div class="instagram_result">'.__('Sending...', $this->domain).'</div>';
        } elseif ($status == "auth-error") {
            echo '<div class="instagram_date"></div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result">'.__('Authorisation error', $this->domain).'</div>';
        } elseif ($status == "image-error") {
            echo '<div class="instagram_date"></div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result">'.__('No thumbnail', $this->domain).'</div>';
            echo '<p><label class="selectit"><input value="1" type="checkbox" name="instagram-not-send">'.__('Do not send this entry', $this->domain).'</label></p>';
        } elseif ($status == "not-sent") {
            echo '<div class="instagram_date"></div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result">'.__('Not sent to instagram', $this->domain).'</div>';
            echo '<p><label class="selectit"><input value="1" type="checkbox" name="instagram-not-send">'.__('Do not send this entry', $this->domain).'</label></p>';
        } elseif ($status == "not-send") {
            echo '<div class="instagram_date"></div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result">'.__('Not sent to instagram', $this->domain).'</div>';
            echo '<p><label class="selectit"><input value="1" type="checkbox" name="instagram-not-send" checked="checked">'.__('Do not send this entry', $this->domain).'</label></p>';
        } else {
            echo '<div class="instagram_date"></div>';
            echo '<span class="instagram_send button" data-id="'.$post->ID.'">'.__('Resend to instagram', $this->domain).'</span>';
            echo '<div class="instagram_result">'.__('Not yet sent...', $this->domain).'</div>';
            echo '<p><label class="selectit"><input value="1" type="checkbox" name="instagram-not-send">'.__('Do not send this entry', $this->domain).'</label></p>';
        }
    }
    
    public function add_column($columns) {
        $arr = array();
        $index = 0;
        
        foreach ($columns as $key => $title) {
            if ($index == 2) {
                $arr['instagram'] = 'Instagram Auto Poster';
            }
            
            $arr[$key] = $title;
            
            $index++;
        }

        return $arr;
    }
    
    public function fill_column($column_name, $post_id) {
        if ($column_name != 'instagram') {
            return;
        }
        
        $status = get_post_meta($post_id, "instagram-send", true);
        
        echo '<span class="instagram">';

        if (is_array($status) && isset($status['status']) && $status['status'] == 'fail') {
            echo "Error: <strong>" . $status['message'] . "</strong>";
        } elseif ((is_array($status) && isset($status['status']) && $status['status'] == 'ok') || $status == "1") {
            echo $this->getPostedDate($post_id);
        } elseif ($status == "sending") {
            echo "Sending...";
        } elseif ($status == "auth-error") {
            echo "Error: <strong>Authorisation error</strong>";
        } elseif ($status == "image-error") {
            echo "Error: <strong>No thumbnail</strong>";
        } else {
            echo "Not yet sent...";
        }

        echo '</span>';
    }
    
    public function getPostedDate($post_id) {
        $time = get_post_meta($post_id, 'instagram-time', 1);
        $code = get_post_meta($post_id, 'instagram-code', 1);
                
        return '<p><a href="https://www.instagram.com/p/'.$code.'/" target="_blank">' . __('Posted on', $this->domain) . ' ' . '('.date("d.m.Y H:i", $time).')</a></p>';
    }
    
    public function onPostSave($post_id) {
        if (isset($_POST['instagram-not-send']) && $_POST['instagram-not-send'] == 1) {
            update_post_meta($post_id, 'instagram-send', 'not-send');
            return;
        }
        
        if (wp_is_post_revision($post_id) || wp_is_post_autosave($post_id)) {
            return;
        }
        
        $post_types = array('post');
        foreach ($this->getSupportPostType() as $post_type) {
            $post_types[] = $post_type['name'];
        }

        if (!in_array(get_post_type($post_id), $post_types)) {
            return;
        }
        
        $isSending = $this->isSending($post_id);
        
        if ($this->getOption('make') == "publish") {
            if (!$isSending) {
                $this->publishToInstagram($post_id);
                return;
            }
        }
        
        if ($this->getOption('make') == "cron") {
            if (!$isSending) {
                update_post_meta($post_id, "instagram-send", "sending");
                wp_schedule_single_event(time() + 1, 'instagram_scheduled', array($post_id));
            }
        }
    }
    
    public function isSending($post_id) {
        $status = get_post_meta($post_id, "instagram-send", true);
        
        if ((is_array($status) && isset($status['status']) && $status['status'] == 'ok') || $status == "1") {
            return true;
        }
        
        return false;
    }
    
    public function ajaxSend() {
        $post_id = $_POST['post_id'];
        
        $result = $this->publishToInstagram($post_id);
        
        if ($result === true) {
            echo json_encode(array(
                'status' => true,
                'html' => $this->getPostedDate($post_id)
            ));
        } else {
            echo json_encode(array(
                'status' => false,
                'error' => $result
            ));
        }

        wp_die();
    }
    
    public function publishToInstagram($post_id) {
        $login = $this->getOption('login');
        $passw = $this->getOption('passw');

        if (empty($login) || empty($passw)) {
            update_post_meta($post_id, "instagram-send", "auth-error");
            return __('Authorisation error', $this->domain);
        }
              
        $Instagram = new Instagram($login, $passw, false);

        try {
            $Instagram->login();
        } catch (InstagramException $e) {
            update_post_meta($post_id, "instagram-send", "auth-error");
            return __('Authorisation error', $this->domain);
        }
        
        try {
            $timestamp = time() + get_option('gmt_offset') * 3600;
            
            if ($this->getOption('crop') == "on") {
                $image = $this->createSquare($this->getImagePath($post_id)); #false or path
            } elseif ($this->getOption('crop') == "with_bg") {
                $image = $this->createSquareWithBacground($this->getImagePath($post_id)); #false or path
            } else {
                $image = $this->createJPG($this->getImagePath($post_id)); #false or path
            }
            
            $caption = $this->getCaption($post_id);
            
            if ($image === false) {
                $image_url = get_post_meta($post_id, "fifu_image_url", true);
                
                if ($image_url) {
                    $tmp_dir = get_temp_dir() . basename($image_url);
    
                    file_put_contents($tmp_dir, file_get_contents($image_url));
    
                    if ($this->getOption('crop') == "on") {
                        $image = $this->createSquare($tmp_dir); #false or path
                    } elseif ($this->getOption('crop') == "with_bg") {
                        $image = $this->createSquareWithBacground($tmp_dir); #false or path
                    } else {
                        $image = $this->createJPG($tmp_dir); #false or path
                    }
                }
            }
            
            if ($image === false) {
                update_post_meta($post_id, "instagram-send", "image-error");
                return __('No thumbnail', $this->domain);
            }

            $result = $Instagram->uploadPhoto($image, $caption);

            if ($result['status'] == 'ok') {
                update_post_meta($post_id, "instagram-send", $result);
                update_post_meta($post_id, "instagram-time", $timestamp);
                update_post_meta($post_id, "instagram-code", $result['media']['code']);
                return true;
            } else {
                update_post_meta($post_id, "instagram-send", $result);
            }
            
            return $result;
        } catch (Exception $e) {
            echo $e->getMessage();
            update_post_meta($post_id, "instagram-exception", $e->getMessage());
        }
    }
    
    public function getImagePath($post_id) {
        return get_attached_file(get_post_thumbnail_id($post_id));
    }
    
    public function createJPG($path) {
        if (!$path) {
            return false;
        }
        $imagedata = getimagesize($path);
        
        $width  = $imagedata[0];
        $height = $imagedata[1];
        $type = $imagedata['mime'];
              
        $tmp_image = imagecreatetruecolor($width, $height);
        
        if ($type == "image/jpeg") {
            $original_image = imagecreatefromjpeg($path);
        } elseif ($type == "image/png") {
            $original_image = imagecreatefrompng($path);
        }
        
        $tmp_dir = get_temp_dir() . basename($path);
        
        imagecopyresampled($tmp_image, $original_image, 0, 0, 0, 0, $width, $height, $width, $height);
        imagejpeg($tmp_image, $tmp_dir, 100);
        
        return $tmp_dir;
    }
    
    public function createSquare($path) {        
        if (!$path) {
            return false;
        }
        $imagedata = getimagesize($path);
        
        $width  = $imagedata[0];
        $height = $imagedata[1];
        $type = $imagedata['mime'];
                
        if ($type == "image/jpeg") {
            $source = imagecreatefromjpeg($path);
        } elseif ($type == "image/png") {
            $source = imagecreatefrompng($path);
        }
        
        $size = min($width, $height);
        
        // Превью
        $thumbs = imagecreatetruecolor($size, $size);
        
        $tmp_dir = get_temp_dir() . basename($path);
        
        // Горизонтальное изображение
        if ($width > $height && $width > $size) {
            imagecopyresampled($thumbs, $source, 0, 0, (($width-$height)/2), 0, $size, $size, $height, $height);
        }
        // Вертикальное изображение
        elseif ($height > $width && $height > $size) {
            imagecopyresampled($thumbs, $source, 0, 0, 0, (($height-$width)/2), $size, $size, $width, $width);
        }
        // Если квадрат
        elseif ($height == $width && $height > $size) {
            imagecopyresampled($thumbs, $source, 0, 0, 0, 0, $size, $size, $width, $width);
        }
        // Изображение меньше размеров стороны
        else {
            $thumbs = $source;
        }
        
        imagejpeg($thumbs, $tmp_dir, 100);
        
        return $tmp_dir;
        
        // Выводим изображение
        #imagejpeg($thumbs);
    }
    
    public function createSquareWithBacground($path) {        
        if (!$path) {
            return false;
        }
        $imagedata = getimagesize($path);
        
        $width  = $imagedata[0];
        $height = $imagedata[1];
        $type = $imagedata['mime'];
                
        if ($type == "image/jpeg") {
            $source = imagecreatefromjpeg($path);
        } elseif ($type == "image/png") {
            $source = imagecreatefrompng($path);
        }
        
        $min = min($width, $height);
        $max = max($width, $height);
        
        // Превью
        $thumbs = imagecreatetruecolor($max, $max);
        $whiteBackground = imagecolorallocate($thumbs, 255, 255, 255); 
        imagefill($thumbs, 0, 0, $whiteBackground);
        
        $x = ($max - $min) / 2;
        
        $tmp_dir = get_temp_dir() . basename($path);

        imagecopyresampled($thumbs, $source, $x, 0, 0, 0, $width, $height, $width, $height);
        
        imagejpeg($thumbs, $tmp_dir, 100);
        
        return $tmp_dir;
        
        // Выводим изображение
        #imagejpeg($thumbs);
    }
    
    public function getCaption($post_id) {
        $this->setPostId($post_id);

        $format = $this->getOption('format');
                
        $result = preg_replace_callback(array('~{(\w+)}~ui', '~{(\w+):(\d+)}~ui'), function($matches) {
            return $this->replaceData($matches[1], $matches[2]);
        }, $format);
        
        return $result;
    }
    
    public function replaceData($tag, $limit = null) {
        $post_id = $this->postId;

        switch ($tag) {
            case 'TITLE':
                return $this->getParam($this->entity_decode(get_the_title($post_id)));
                break;
            case 'URL':
                return $this->getParam($this->getUrl($post_id));
                break;
            case 'EXCERPT':
                return $this->getParam($this->getExcerpt($post_id));
                break;
            case 'CONTENT':
                return $this->getParam($this->getContent($post_id));
                break;
            case 'AUTHORNAME':
                return $this->getParam(get_the_author_meta('display_name', get_post_field('post_author', $post_id)));
                break;
            case 'SITENAME':
                return $this->getParam(get_bloginfo('name'));
                break;
            case 'CATS':
                return $this->getParam($this->getCategory($post_id, $limit));
                break;
            case 'HCATS':
                return $this->getParam($this->getHcats($post_id, $limit));
                break;
            case 'TAGS':
                return $this->getParam($this->getTags($post_id, $limit));
                break;
            case 'HTAGS':
                return $this->getParam($this->getHtags($post_id, $limit));
                break;
            case 'SKU':
                return $this->getParam($this->getMeta($post_id, '_sku'));
                break;
            case 'PRICE':
                return $this->getParam($this->getPrice($post_id));
                break;
        }
        
        $taxonomies = get_taxonomies(array('public' => true, '_builtin' => false,), 'objects');
        
        foreach ($taxonomies as $name => $taxonomy) {
            $upper = mb_strtoupper($name);
            
            if ($tag == $upper) {
                return $this->getParam($this->getTaxonomy($post_id, $name, $limit));
            }
            if ($tag == "HTAGS_$upper") {
                return $this->getParam($this->getHTaxonomy($post_id, $name, $limit));
            }
        }
    }
    
    public function getMeta($post_id, $name) {
        $meta = get_post_meta($post_id, $name, true);
        $post = $_POST[$name];
        return $meta ? $meta : $post;
    }
    
    public function getPrice($post_id) {
        $regular = get_post_meta($post_id, '_regular_price', true);
        $sale = get_post_meta($post_id, '_sale_price', true);
        
        $post_regular = $_POST['_regular_price'];
        $post_sale = $_POST['_sale_price'];
        
        $regular = $regular ? $regular : $post_regular;
        $sale = $sale ? $sale : $post_sale;
        
        return $sale ? $sale : $regular;
    }
    
    public function getUrl($post_id) {        
        return $this->getOption('shortlink') == "on" ? wp_get_shortlink($post_id) : get_permalink($post_id);
    }
    
    public function getExcerpt($post_id) {
        global $post;  
        
        $save_post = $post;
        $post = get_post($post_id);
        setup_postdata($post);
        $output = $this->entity_decode(get_the_excerpt());
        $post = $save_post;
        return trim($this->strip_all_shortcodes($output));
    }
    
    public function getContent($post_id) {
        return strip_tags(get_post_field('post_content', $post_id));
    }
    
    public function entity_decode($text) {
        return html_entity_decode($text, ENT_QUOTES, 'UTF-8');
    }
    
    public function strip_all_shortcodes($text){
        $text = preg_replace("/\[[^\]]+\]/", '', $text); #strip shortcode
        return $text;
    }
    
    public function getParam($value) {
        return $value ? $value : '';
    }
    
    public function setSeparator($tags) {
        $tags = explode(',', $tags);
        if ($this->getOption('separator') == 'remove') {
            $tags = str_replace(' ', '', $tags);
            return implode(' ', $tags);
        }
        if ($this->getOption('separator') == 'dash') {
            $tags = str_replace(' ', '-', $tags);
            return implode(' ', $tags);
        }
        if ($this->getOption('separator') == 'underscore') {
            $tags = str_replace(' ', '_', $tags);
            return implode(' ', $tags);
        }
        if ($this->getOption('separator') == 'not') {
            return implode(' ', $tags);
        }
    }
    
    public function getTags($post_id, $limit) {
        $arr = array();
        $tags = get_the_tags($post_id);

        if ($tags) {
            foreach ($tags as $index => $tag) {
                if ($limit) {
                    if ($index >= $limit) continue;
                }
                $arr[] = $tag->name;
            }
        }

        return $this->setSeparator(implode(',', $arr));
    }
    
    public function getCategory($post_id, $limit) {
        $arr = array();
        $category = get_the_category($post_id);
        
        if ($category) {
            foreach ($category as $index => $tag) {
                if ($limit) {
                    if ($index >= $limit) continue;
                }
                $arr[] = $tag->name;
            }
        }
        
        return $this->setSeparator(implode(',', $arr));
    }
    
    public function getHcats($post_id, $limit) {
        $arr = array();
        $category = get_the_category($post_id);
        
        if ($category) {
            foreach ($category as $cat) {
                if ($this->getOption('separator') == 'remove') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '', $cat->name));
                }
                if ($this->getOption('separator') == 'dash') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '-', $cat->name));
                }
                if ($this->getOption('separator') == 'underscore') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '_', $cat->name));
                }
                if ($this->getOption('separator') == 'not') {
                    $arr[] = $cat->name;
                }
            }
        }
        
        $htags = "";
        foreach ($arr as $index => $tag) {
            if ($limit) {
                if ($index >= $limit) continue;
            }
            $htags .= " #" . $tag;
        }

        return trim(trim($htags, ','));
    }
    
    public function getHtags($post_id, $limit) {
        $arr = array();
        $tags = get_the_tags($post_id);

        if ($tags) {
            foreach ($tags as $tag) {
                if ($this->getOption('separator') == 'remove') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '', $tag->name));
                }
                if ($this->getOption('separator') == 'dash') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '-', $tag->name));
                }
                if ($this->getOption('separator') == 'underscore') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '_', $tag->name));
                }
                if ($this->getOption('separator') == 'not') {
                    $arr[] = $tag->name;
                }
            }
        }

        $htags = "";
        foreach ($arr as $index => $tag) {
            if ($limit) {
                if ($index >= $limit) continue;
            }
            $htags .= " #" . $tag;
        }
        
        return trim(trim($htags, ' '));
    }
    
    public function getTaxonomy($post_id, $taxonomy, $limit) {
        $arr = array();
        $category = get_the_terms($post_id, $taxonomy);
        
        if ($category) {
            foreach ($category as $index => $tag) {
                if ($limit) {
                    if ($index >= $limit) continue;
                }
                $arr[] = $tag->name;
            }
        }
        
        return $this->setSeparator(implode(',', $arr));
    }
    
    public function getHTaxonomy($post_id, $taxonomy, $limit) {
        $arr = array();
        $tags = get_the_terms($post_id, $taxonomy);
        
        if ($tags) {
            foreach ($tags as $tag) {
                if ($this->getOption('separator') == 'remove') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '', $tag->name));
                }
                if ($this->getOption('separator') == 'dash') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '-', $tag->name));
                }
                if ($this->getOption('separator') == 'underscore') {                    
                    $arr[] = mb_strtolower(str_replace(' ', '_', $tag->name));
                }
                if ($this->getOption('separator') == 'not') {
                    $arr[] = $tag->name;
                }
            }
        }
        
        $htags = "";
        foreach ($arr as $index => $tag) {
            if ($limit) {
                if ($index >= $limit) continue;
            }
            $htags .= " #" . $tag;
        }
        
        return trim(trim($htags, ' '));
    }
    
    public function admincss() {
        echo <<<HTML
            <style type='text/css'>
                .instagram_result {
                    font-size: 16px;
                    margin: 10px 0 0 0;
                }
            </style>
HTML;
    }

    public function adminjs() {
        echo '
            <script type="text/javascript">
            jQuery(document).ready(function($) {
    
                jQuery(".instagram_send").bind("click", function(){
                    var post_id = jQuery(this).data("id");
                    var data = {
                        action: "instagram_send",
                        post_id: post_id
                    };
    
                    jQuery(".instagram_result").html("'.__('Sending...', $this->domain).'");
                    jQuery.post(ajaxurl, data, function(json) {
                        if (json.status) {
                            jQuery(".instagram_date").html(json.html);
                            jQuery(".instagram_result").html("'.__('Successfully sent', $this->domain).'");
                        } else {
                            if (json.error) var error = json.error;
                            if (json.error.message) var error = json.error.message;
                            jQuery(".instagram_result").html("'.__('Error:', $this->domain).' " + error);
                        }
                    }, "json");
                });
                
            });
            </script>
        ';
    }
    
    public function display_input_field($args) {
        extract( $args );
        
        $option = $this->getOption($id);
        switch ( $type ) {
            case 'text':
                echo sprintf(
                    "<input class='regular-text' type='text' id='%s' required='required' name='%s[%s]' value='%s' />", $id, $this->settingsGroup, $id, esc_attr(stripslashes($option))
                );
                echo $desc ? "<br /><span class='description'>$desc</span>" : "";
                break;
            case 'password':
                echo sprintf(
                    "<input class='regular-text' type='password' id='%s' required='required' name='%s[%s]' value='%s' />", $id, $this->settingsGroup, $id, esc_attr(stripslashes($option))
                );
                echo $desc ? "<br /><span class='description'>$desc</span>" : "";
                break;
            case 'textarea':
                echo sprintf(
                    "<textarea class='code large-text' cols='50' rows='10' type='text' id='%s' name='%s[%s]'>%s</textarea>", $id, $this->settingsGroup, $id, esc_attr(stripslashes($option))
                );
                echo $desc ? "<br /><span class='description'>$desc</span>" : "";
                break;
            case 'checkbox':
                $checked = ($option == 'on') ? " checked='checked'" :  '';
                echo sprintf(
                    "<label><input type='checkbox' id='%s' name='%s[%s]' %s /> ", $id, $this->settingsGroup, $id, $checked
                );
                echo $desc ? $desc : "";
                echo "</label>";
                break;
            case 'select':
                echo sprintf(
                    "<select id='%s' name='%s[%s]'>", $id, $this->settingsGroup, $id
                );
                foreach($vals as $v => $l){
                    $selected = ($option == $v) ? "selected='selected'" : '';
                    echo "<option value='$v' $selected>$l</option>";
                }
                echo $desc ? $desc : "";
                echo "</select>";
                break;
            case 'radio':
                echo "<fieldset>";
                foreach($vals as $v=>$l){
                    $checked = ($option == $v || !$option) ? "checked='checked'" : '';
                    echo sprintf(
                        "<label><input type='radio' name='%s[%s]' value='%s' %s />%s</label><br />", $this->settingsGroup, $id, $v, $checked, $l
                    );
                }
                echo "</fieldset>";
                break;
        }
    }
}